//
//  xAdLocalSearchRequest.m
//  xAd SDK
//
//  Created by Stephen Anderson on 5/15/13.
//  Copyright (c) 2013 xAd Inc. All rights reserved.
//


#import "xAdLocalSearchRequest.h"
#import "xAdLocalSearchResults.h"


#import "xAdSdk.h"
#import "xAdUtils.h"


@implementation xAdLocalSearchRequest


- (id) init {

    self = [super init];
    
    if (self)
    {
        xAdSdk *sdk = [xAdSdk sharedInstance];

        CLLocation* currentLocation = [xAdSdk currentLocation];

        NSDictionary *defaultValues = @{
                                        xAdParameterGeoLat:[NSString stringWithFormat:@"%f", currentLocation.coordinate.latitude],
                                        xAdParameterGeoLong:[NSString stringWithFormat:@"%f", currentLocation.coordinate.longitude],
                                        xAdParameterPublisherKey: sdk.searchPublisherKey,
                                        xAdParameterVersion: @"1.1",
                                        xAdParameterAdFormat: @"json",
                                        xAdParameterDeviceCarrier: [xAdUtils carrier],
                                        xAdParameterDeviceUserAgent: [xAdUtils userAgent],
                                        xAdParameterUserId: [xAdUtils advertisingIdentifier],
                                        xAdParameterUserIdType: @"IDFA|RAW",
                                        xAdParameterUserDoNotTrack: [xAdUtils isDoNotTrackEnabled],
                                        xAdParameterPublisherType:@"app",
                                        xAdParameterSearchListings:@"ad",
                                        xAdParameterSearchOrganicsStartIndex:@"0",
                                        xAdParameterSearchOrganicsNumberOfItems:@"12"
                                        };

        [self.fields addEntriesFromDictionary: defaultValues];
    }
    
    return self;
}


#pragma mark - xAdRequest

- (xAdRequestType) requestType
{
    return RequestTypeLocalSearch;
}

- (NSString*) RESTCommand
{
    return @"local";
}

- (NSString*) notificationEventName {
    return xAdEventSearchCompleted;
}

- (xAdResponse*) createResponseFromJson:(NSData*) jsonData
{
    return [[xAdLocalSearchResults alloc] initWithData:jsonData];
}


#pragma mark - xAdLocalSearchRequest

- (void) setTrafficSource:(NSString*)trafficSource {
    [self.fields setObject:trafficSource forKey:xAdParameterPublisherTrafficSource];
}

- (void) setSort:(SortType)sortType
{
    id value = nil;
    
    switch(sortType)
    {
        case SortTypeRelevance:
            value = @"relevance"; break;
            
        case SortTypeDistance:
            value = @"dist"; break;
            
        case SortTypeAlphabetically:
            value = @"alpha"; break;
            
        case SortTypeReview:
            value = @"review"; break;
            
        default:
            break;
    }
    
    if (value) {
        [self.fields setObject:value forKey:xAdParameterSearchOrganicsSort];
    }
}


- (void) setStartIndex:(NSUInteger)startIndex
{
    [self.fields setObject:[NSString stringWithFormat:@"%d", startIndex] forKey:xAdParameterSearchOrganicsStartIndex];
}


- (void) setSearchQuery:(NSString*)query
{
    if (query != nil && [query length] > 0) {
        [self.fields setObject:query forKey:xAdParameterAdInput];
    }
}


- (void) setUserGender:(Gender)gender {
    id value = (gender == GenderMale) ? @"m" : @"f";
    [self.fields setObject:value forKey:xAdParameterUserGender];
}


- (void) setUserAge:(NSUInteger)age {
    
#warning Sponsored listing will not be served to under 13 year old aged individuals.
    [self.fields setObject:[NSString stringWithFormat:@"%d", age] forKey:xAdParameterUserAge];
}


- (void) setLocation:(CLLocation*)location
{
    NSAssert(location != nil, @"Invalid Argument: location cannot be nil.");
    
    [self.fields removeObjectForKey:xAdParameterGeoCityState];
    [self.fields removeObjectForKey:xAdParameterGeoCountry];
    [self.fields removeObjectForKey:xAdParameterSearchStreetName];
    [self.fields removeObjectForKey:xAdParameterSearchStreetCity];
    [self.fields removeObjectForKey:xAdParameterSearchStreetNumber];
    
    [self.fields setObject:[NSString stringWithFormat:@"%f", location.coordinate.longitude] forKey:xAdParameterGeoLong];
    [self.fields setObject:[NSString stringWithFormat:@"%f", location.coordinate.latitude] forKey:xAdParameterGeoLat];
}


- (void) setStreetNumber:(NSString*)number andStreet:(NSString*)street
{
    // Flexible property setter allows user to provide what they have only
    // Example:
    // [self setStreetNumber:nil andStreet:@"Spear street" ];
    // [self setCityState:@"San Francisco"];
    
    
    // Remove LAT & LONG as we will be using the STREET INFO
    [self.fields removeObjectForKey:xAdParameterGeoLong];
    [self.fields removeObjectForKey:xAdParameterGeoLat];
    
    // Only add elements which are available
    if (number != nil)
        [self.fields setObject:number forKey:xAdParameterSearchStreetNumber];
    
    if (street != nil)
        [self.fields setObject:street forKey:xAdParameterSearchStreetName];
}


- (void) setCityStateOrZip:(NSString*)value
{
    NSAssert(value != nil, @"Invalid Argument: location cannot be nil.");
    NSAssert([value length] > 0, @"Invalid Argument: location cannot be empty.");
    
    // Remove LAT & LONG as we will be using the LOCATION INFO
    [self.fields removeObjectForKey:xAdParameterGeoLat];
    [self.fields removeObjectForKey:xAdParameterGeoLong];
    
    [self.fields setObject:value forKey:xAdParameterGeoCityState];
    [self.fields setObject:@"US" forKey:xAdParameterGeoCountry];
}


- (void) setCityStateOrZip:(NSString*)value andCountry:(NSString*)country
{
    NSAssert(value != nil, @"Invalid Argument: location cannot be nil.");
    NSAssert([value length] > 0, @"Invalid Argument: location cannot be empty.");
    NSAssert(country != nil, @"Invalid Argument: country cannot be nil.");
    NSAssert([country length] > 0, @"Invalid Argument: country cannot be empty.");
    
    // Remove LAT & LONG as we will be using the LOCATION INFO
    [self.fields removeObjectForKey:xAdParameterGeoLat];
    [self.fields removeObjectForKey:xAdParameterGeoLong];
    
    [self.fields setObject:value forKey:xAdParameterGeoCityState];
    [self.fields setObject:country forKey:xAdParameterGeoCountry];
}


- (void) setRadius:(NSUInteger)radius
{
    [self.fields setObject:[NSString stringWithFormat:@"%d", radius] forKey:xAdParameterSearchOrganicsFilterRadius];
}


- (void) setRequestId:(NSString*)requestId {
    [self.fields setObject:requestId forKey:xAdParameterSearchRequestId];
}



- (void) enableOrganicListings {
    [self.fields removeObjectForKey:xAdParameterSearchListings];
}



- (void) setNumberOfOrganicResults:(NSUInteger)count {
    [self.fields setObject:[NSString stringWithFormat:@"%d", count] forKey:xAdParameterSearchOrganicsNumberOfItems];
}






@end
